﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO;
using System.Diagnostics;
using HtmlWpfRtf.Common;

namespace HtmlWpfRtf {
    /// <summary>
    /// Interaktionslogik für MyFileList2.xaml
    /// </summary>
    public partial class MyFileList2 : UserControl, ISetMainWindows {

        String path;
        IMiddleware middleware;
        Window parent;
        private FileSystemWatcher fsw;

        private bool browserVisible = true;

        public MyFileList2() {
            InitializeComponent();
        }

        public void init(Window parent, IMiddleware middleware) {

            
            path = SingVar.LastPath;
            if (!path.EndsWith("\\")) path += "\\";

            this.middleware = middleware;
            this.parent = parent;

            path = SingVar.LastPath;
            if (!path.EndsWith("\\")) path += "\\";

            fsw = new FileSystemWatcher();
            fsw.EnableRaisingEvents = false;
            // Events definieren
            fsw.Changed += new FileSystemEventHandler(FSW_Changed);
            fsw.Created += new FileSystemEventHandler(FSW_Changed);
            fsw.Deleted += new FileSystemEventHandler(FSW_Changed);
            fsw.Renamed += new RenamedEventHandler(FSW_Changed);

            setFileWatcher(path);

            //lPath = new Label();
            tPath.Text = "Path";

            createFileList();
            insertLogicalDrives();
        }

        public void setColors() {
            listbox.Foreground = new SolidColorBrush(Color.FromRgb((byte)SingVar.FontcolorRed, (byte)SingVar.FontcolorGreen, (byte)SingVar.FontcolorBlue));
            listbox.Background = new SolidColorBrush(Color.FromRgb((byte)SingVar.BGcolorRed, (byte)SingVar.BGcolorGreen, (byte)SingVar.BGcolorBlue));

        }

        public void editorIsModfied() {
            throw new NotImplementedException();
        }

        public String getPath() {
            return path;
        }


        delegate void showFilesInListViewCallback(string actualPath);

        void FSW_Changed(object sender, FileSystemEventArgs e) {
            try {
                Application.Current.Dispatcher.Invoke(new Action(() => { createFileList(); }));
            }
            catch (Exception e2) {
                Basis.ErrorMsg("Fehler beim Aktualisieren der Dateienliste\r\n"+e2.ToString(), "Hinweis");
            }
            
        }

        private void setFileWatcher(String actualPath) {
            if (actualPath.Length > 0) {
                fsw.Path = actualPath;
                fsw.Filter = "*.*";
                fsw.EnableRaisingEvents = true;
            }
        }  // setFileWatcher




        public void updatePath() {
            path = SingVar.LastPath;
            if (!path.EndsWith("\\")) path += "\\";
            path = SingVar.LastPath;
            if (!path.EndsWith("\\")) path += "\\";
            createFileList();
            setFileWatcher(path);
        }

        public void update() {
            createFileList();
        }

        // setzt den internen Browser
        public void setFile2Browser(String filename) {
            if (File.Exists(filename)) {
                String s = "file:///" + filename;
                if (browserVisible) webbrowser.Navigate("file:///" + filename);
            }
        }

        public void setFontsize(int size) {
            listbox.FontSize = size;
            comboBox1.FontSize = size;
            tPath.FontSize = size;
            laufwerk.FontSize = size;
        }  

        public void setBrowserVisible(bool visible) {
            browserVisible = visible;
            if (visible) {
                grid.RowDefinitions[3].Height = new GridLength(1, GridUnitType.Star);
            }
            else {
                grid.RowDefinitions[3].Height = new GridLength(0);
            }
        }

        private void listbox_MouseDoubleClick(object sender, MouseButtonEventArgs e) {
            FileObject fobj = (FileObject)listbox.SelectedItem;
            if (fobj != null) {
                if (fobj.isParent) {
                    DirectoryInfo dir = Directory.GetParent(path);
                    path = dir.Parent.FullName;
                    if (!path.EndsWith("\\")) path += "\\";
                    SingVar.LastPath = path;
                    updatePath();
                    // createFileList();
                }
                else if (fobj.isDir) {
                    path += fobj.name + "\\";
                    SingVar.LastPath = path;
                    // createFileList();
                    updatePath();
                }
                else {
                    middleware.insertFile(path + fobj.name);
                }
            }

        }

        private void listbox_KeyDown(object sender, KeyEventArgs e) {
            if (Keyboard.IsKeyDown(Key.Delete)) {
                deletFile();
                e.Handled = true;
                return;
            }
            if (Keyboard.IsKeyDown(Key.F2)) {
                renameFileF2();
                e.Handled = true;
                return;
            }
            if (Keyboard.IsKeyDown(Key.Insert)) {
                newFolder();
                e.Handled = true;
                return;
            }
        }

        public void newFolder() {
            if (!path.Equals("")) {                
                if (!path.EndsWith("\\")) path += "\\";
                WReadString dialog = new WReadString(parent, "Neuer Ordner", "Ordnername");
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    String newDir = path + dialog.tInput.Text.Trim();
                    System.IO.Directory.CreateDirectory(newDir);
                    //createFileList();
                }
            }
        } // NewFolder

        private void MnInsertDirectory_Click(Object sender, RoutedEventArgs e) {
            newFolder();
        }

        private void MnExplorer_Click(Object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Explorer.exe";
            if (path.Length > 0)
                Prog.StartInfo.Arguments = "/e," + path;
            else
                Prog.StartInfo.Arguments = "/e,";
            Prog.Start();
        }

        private void MnDos_Click(Object sender, RoutedEventArgs e) {
            dos();
        }

        public void dos() {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "CMD.exe";
            Prog.StartInfo.Arguments = "";
            Prog.StartInfo.WorkingDirectory = path;
            Prog.Start();

        }



        private void deletFile() {
            int i = listbox.SelectedIndex;
            if (i >= 0) {
                FileObject fobj = (FileObject)listbox.SelectedItem;
                String filename = path + fobj.name;
                String caption = "";
                if (Directory.Exists(filename)) {
                    caption = "Do you really want to delete completely this directory ?\r\n";
                }
                if (File.Exists(filename)) {
                    caption = "Do you really want to delete this file(s) ?\r\n";
                }
                if (Basis.readYesNoCancel(caption + fobj.name, "Delete") == MessageBoxResult.Yes) {
                    
                    if (File.Exists(filename)) {
                        File.Delete(filename);
                        filename = Basis.changeFilenameExt(path + fobj.name, "css"); ;
                        if (File.Exists(filename)) {
                            File.Delete(filename);
                        }
                        //createFileList();
                    }
                    if (Directory.Exists(filename)) {
                        Directory.Delete(filename, true);
                    }
                    
                }
            }
        }

        public void renameFileF2() {
            int i = listbox.SelectedIndex;
            if (i >= 0) {
                FileObject fobj = (FileObject)listbox.SelectedItem;
                String fullOldFilename = path + fobj.name;
                String filename;
                System.IO.FileInfo fileinfo = new FileInfo(fullOldFilename);
                bool isDir = false;
                if (Directory.Exists(fullOldFilename)) {
                    isDir = true;
                }

                Basis.splitFilename(fullOldFilename, out path, out filename);
                WRenameWindow dialog = new WRenameWindow(parent, filename, isDir);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    String newfile = dialog.tNewname.Text.Trim();
                    String newFullFilename;
                    if (newfile.IndexOf("\\") >= 0)
                        newFullFilename = newfile;
                    else
                        newFullFilename = path + newfile;
                    if (isDir) {
                        DirectoryInfo di = new DirectoryInfo(fullOldFilename);
                        if (fullOldFilename.ToLower().Equals(newFullFilename.ToLower())) {
                            di.MoveTo(System.IO.Path.Combine(path, "1234!!!!!!666"));
                            di.MoveTo(System.IO.Path.Combine("1234!!!!!!666", newFullFilename));
                        }
                        else {
                            di.MoveTo(newFullFilename);
                        }
                    }
                    else {
                        if (fullOldFilename.ToLower().Equals(newFullFilename.ToLower())) {
                            System.IO.File.Move(path, "1234!!!!!!666");
                            System.IO.File.Move("1234!!!!!!666", newFullFilename);
                        }
                        else {
                            System.IO.File.Move(fullOldFilename, newFullFilename);
                        }
                    }
                    
                    //createFileList();
                }
            }
        }


        // string[] dirs = Directory.GetFiles(@"c:\", "c*");
        private void createFileList() {
            listbox.Items.Clear();
            List<FileObject> liste = new List<FileObject>();

            // nicht im root
            if (path.Length > 3) {
                listbox.Items.Add(new FileObject("..", false, true));
            }

            tPath.Text = path; //  Basis.getLastPath(path);
            tPath.ToolTip = path;
            // lPath.ToolTip = Basis.getLastPath(path);

            string[] subdirectoryEntries = Directory.GetDirectories(path);
            foreach (string fileName in subdirectoryEntries) {
                liste.Add(new FileObject(Basis.getFilename(fileName), true, false));
            }
            string[] fileEntries = Directory.GetFiles(path, "*.xhtml");
            foreach (string fileName in fileEntries) {
                liste.Add(new FileObject(Basis.getFilename(fileName), false, false));
            }

            fileEntries = Directory.GetFiles(path, "*.html");
            foreach (string fileName in fileEntries) {
                liste.Add(new FileObject(Basis.getFilename(fileName), false, false));
            }

            fileEntries = Directory.GetFiles(path, "*.php");
            foreach (string fileName in fileEntries) {
                liste.Add(new FileObject(Basis.getFilename(fileName), false, false));
            }

            fileEntries = Directory.GetFiles(path, "*.js");
            foreach (string fileName in fileEntries) {
                liste.Add(new FileObject(Basis.getFilename(fileName), false, false));
            }
            fileEntries = Directory.GetFiles(path, "*.jsp");
            foreach (string fileName in fileEntries) {
                liste.Add(new FileObject(Basis.getFilename(fileName), false, false));
            }
            fileEntries = Directory.GetFiles(path, "*.css");
            foreach (string fileName in fileEntries) {
                liste.Add(new FileObject(Basis.getFilename(fileName), false, false));
            }
            liste.Sort();

            foreach (FileObject fo in liste) { 
                listbox.Items.Add(fo);
            }
            listbox.AlternationCount = 2;

        } // createFileList


        void insertLogicalDrives() {
            try {
                comboBox1.Items.Clear();
                // string[] drives = System.IO.Directory.GetLogicalDrives();
                DriveInfo[] drives = DriveInfo.GetDrives();

                foreach (DriveInfo drv in drives) {

                    switch (drv.DriveType) {
                        case DriveType.CDRom:
                            //cbDrivelist.Items.Add(drv.Name + " CD/DVD");
                            break;
                        case DriveType.Fixed:
                            comboBox1.Items.Add(drv.Name + " Festplatte");
                            break;
                        case DriveType.Network:
                            comboBox1.Items.Add(drv.Name + " Netzwerk");
                            break;
                        case DriveType.Ram:
                            comboBox1.Items.Add(drv.Name + " RAM");
                            break;
                        case DriveType.Removable:
                            comboBox1.Items.Add(drv.Name + " USB");
                            break;
                        case DriveType.Unknown:
                            // cbDrivelist.Items.Add(drv.Name + " ??");
                            break;
                    }
                }
                comboBox1.Items.Add("Desktop");
                comboBox1.Items.Add("Meine Dokumente");
                comboBox1.Items.Add("Downloads");
            }
            catch (System.IO.IOException) {
                System.Console.WriteLine("An I/O error occurs.");
            }
            catch (System.Security.SecurityException) {
                System.Console.WriteLine("The caller does not have the required permission.");
            }
        }


        private void comboBox1_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            String sLw = (String)comboBox1.SelectedItem;
            if (sLw.Equals("Desktop")) {
                path = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
            }
            else if (sLw.Equals("Meine Dokumente")) {
                path = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            }
            else if (sLw.Equals("Downloads")) {
                path = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
                path.Replace("Desktop", "Downloads");
            }
            else {
                path = (String)sLw.Substring(0, 3);  // C:\\
                // path = sLw;
            }
            if (!path.EndsWith("\\")) path += "\\";
            SingVar.LastPath = path;
            createFileList();
        }

    }
}
